package de.intevation.flys.artifacts.states;

import java.util.Map;

import org.apache.log4j.Logger;

import org.w3c.dom.Element;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.artifacts.common.utils.XMLUtils;

import de.intevation.artifactdatabase.ProtocolUtils;
import de.intevation.artifactdatabase.data.StateData;

import de.intevation.flys.model.Gauge;
import de.intevation.flys.model.River;
import de.intevation.flys.model.Wst;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.WstFactory;
import de.intevation.flys.artifacts.resources.Resources;

/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class WQSelect extends RangeState {

    /** The logger used in this class.*/
    private static Logger logger = Logger.getLogger(WQSelect.class);


    /** The default step width for Qs.*/
    public static final String DEFAULT_STEP_Q = "50";

    /** The default step width for Qs.*/
    public static final String DEFAULT_STEP_W = "30";

    /** The name of the 'mode' field. */
    public static final String WQ_MODE = "wq_mode";

    /** The name of the 'from' field. */
    public static final String WQ_FROM = "wq_from";

    /** The name of the 'to' field. */
    public static final String WQ_TO = "wq_to";

    /** The name of the 'step' field. */
    public static final String WQ_STEP = "wq_step";

    /**
     * The default constructor that initializes an empty State object.
     */
    public WQSelect() {
    }

    protected Element createData(
        XMLUtils.ElementCreator cr,
        Artifact    artifact,
        StateData   data,
        CallContext context)
    {
        Element select = ProtocolUtils.createArtNode(
            cr, "select", null, null);

        cr.addAttr(select, "name", data.getName(), true);

        Element label = ProtocolUtils.createArtNode(
            cr, "label", null, null);

        Element choices = ProtocolUtils.createArtNode(
            cr, "choices", null, null);

        label.setTextContent(Resources.getMsg(
            context.getMeta(),
            data.getName(),
            data.getName()));

        select.appendChild(label);

        return select;
    }


    protected Element[] createItems(
        XMLUtils.ElementCreator cr,
        Artifact    artifact,
        String      name,
        CallContext context)
    {
        // TODO Insert correct min/max values!
        double[] minmaxW = determineMinMaxW(artifact);
        double[] minmaxQ = determineMinMaxQ(artifact);

        if (name.equals("wq_from")) {
            Element minW = createItem(
                cr, new String[] {"minW", new Double(minmaxW[0]).toString()});
            Element minQ = createItem(
                cr, new String[] {"minQ", new Double(minmaxQ[0]).toString()});
            return new Element[] { minW, minQ };
        }
        else if (name.equals("wq_to")) {
            Element maxW = createItem(
                cr, new String[] {"maxW", new Double(minmaxW[1]).toString()});
            Element maxQ = createItem(
                cr, new String[] {"maxQ", new Double(minmaxQ[1]).toString()});
            return new Element[] { maxW, maxQ };
        }
        else {
            Element stepW = createItem(cr, new String[] {"stepW", DEFAULT_STEP_W});
            Element stepQ = createItem(cr, new String[] {"stepQ", DEFAULT_STEP_Q});
            return new Element[] { stepW, stepQ };
        }
    }


    protected Element createItem(XMLUtils.ElementCreator cr, Object obj) {
        Element item  = ProtocolUtils.createArtNode(cr, "item", null, null);
        Element label = ProtocolUtils.createArtNode(cr, "label", null, null);
        Element value = ProtocolUtils.createArtNode(cr, "value", null, null);

        String[] arr = (String[]) obj;

        label.setTextContent(arr[0]);
        value.setTextContent(arr[1]);

        item.appendChild(label);
        item.appendChild(value);

        return item;
    }


    protected String getUIProvider() {
        return "wq_panel";
    }


    /**
     * Determines the min and max W value for the current gauge. If no min and
     * max values could be determined, this method will return
     * [Double.MIN_VALUE, Double.MAX_VALUE].
     *
     * @param artifact The FLYSArtifact.
     *
     * @return the min and max W values for the current gauge.
     */
    protected double[] determineMinMaxW(Artifact artifact) {
        logger.debug("WQSelect.determineCurrentGauge");

        Gauge    gauge   = ((FLYSArtifact) artifact).getGauge();
        double[] minmaxW = gauge != null ? gauge.determineMinMaxW() : null;

        double minW = minmaxW != null ? minmaxW[0] : Double.MIN_VALUE;
        double maxW = minmaxW != null ? minmaxW[1] : Double.MAX_VALUE;

        return new double[] { minW, maxW };
    }


    /**
     * Determines the min and max Q value for the current gauge. If no min and
     * max values could be determined, this method will return
     * [Double.MIN_VALUE, Double.MAX_VALUE].
     *
     * @param artifact The FLYSArtifact.
     *
     * @return the min and max Q values for the current gauge.
     */
    protected double[] determineMinMaxQ(Artifact artifact) {
        logger.debug("WQSelect.determineMinMaxQ");

        FLYSArtifact flysArtifact = (FLYSArtifact) artifact;

        River    river = flysArtifact.getRiver();
        Gauge    gauge = flysArtifact.getGauge();
        Wst      wst   = WstFactory.getWst(river);

        double[] minmaxQ = gauge != null
            ? wst.determineMinMaxQ(gauge.getRange())
            : null;

        double minQ = minmaxQ != null ? minmaxQ[0] : Double.MIN_VALUE;
        double maxQ = minmaxQ != null ? minmaxQ[1] : Double.MAX_VALUE;

        return new double[] { minQ, maxQ };
    }


    @Override
    public boolean validate(Artifact artifact, CallContext context)
    throws IllegalArgumentException
    {
        logger.debug("WQSelect.validate");

        Map<String, StateData> data = getData();

        String mode = (String) data.get(WQ_MODE).getValue();
        logger.debug("WQ Mode: " + mode);

        String fromStr = (String) data.get(WQ_FROM).getValue();
        String toStr   = (String) data.get(WQ_TO).getValue();
        String stepStr = (String) data.get(WQ_STEP).getValue();

        double from = Double.parseDouble(fromStr);
        double to   = Double.parseDouble(toStr);
        double step = Double.parseDouble(stepStr);

        try {
            if (mode != null && mode.trim().toLowerCase().equals("w")) {
                return validateW(artifact, context, from, to, step);
            }
            else if (mode != null && mode.trim().toLowerCase().equals("q")) {
                return validateQ(artifact, context, from, to, step);
            }
            else {
                throw new IllegalArgumentException("error_feed_invalid_wq_mode");
            }
        }
        catch (NumberFormatException nfe) {
            throw new IllegalArgumentException("error_feed_number_format");
        }
    }


    /**
     * Validates the inserted W values.
     *
     * @param artifact The owner artifact.
     * @param context The CallContext
     * @param from The lower value of the W range.
     * @param to The upper value of the W range.
     * @param step The step width.
     *
     * @return true, if everything was fine, otherwise an exception is thrown.
     */
    protected boolean validateW(
        Artifact    artifact,
        CallContext context,
        double from,
        double to,
        double step)
    throws    IllegalArgumentException
    {
        logger.debug("WQSelect.validateW");

        double[] minmaxW = determineMinMaxW(artifact);

        return validateRange(minmaxW[0], minmaxW[1], from, to, step);
    }


    /**
     * Validates the inserted Q values.
     *
     * @param artifact The owner artifact.
     * @param context The CallContext
     * @param from The lower value of the Q range.
     * @param to The upper value of the Q range.
     * @param step The step width.
     *
     * @return true, if everything was fine, otherwise an exception is thrown.
     */
    protected boolean validateQ(
        Artifact    artifact,
        CallContext context,
        double from,
        double to,
        double step)
    throws    IllegalArgumentException
    {
        logger.debug("WQSelect.validateQ");

        double[] minmaxQ = determineMinMaxQ(artifact);

        return validateRange(minmaxQ[0], minmaxQ[1], from, to, step);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
